package abstractions;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * <b> Implements </b> {@link KeyListener}<br><br>
 * Specification of a simple paddle controller for use in a video game.<br>
 * Typing a character on the keyboard changes direction of movement of the Paddle. <br>
 * For teaching MVC design pattern.
 * 
 * @version 1.0.0
 * @author J Paul Gibson
 */


public abstract class PaddleControllerAbstraction implements  KeyListener{
	
	/**
	 * Should not react to key presses
	 * @param e is the event generated by the keyboard
	 */
	public void keyPressed(KeyEvent e){};

	/**
	 * Should not react to key releases
	 * @param e is the event generated by the keyboard
	 */
	public void keyReleased(KeyEvent e){};

	/**
	 * Should react to key typing (press and release)
	 * @param e is the event generated by the keyboard
	 */
	public abstract void keyTyped(KeyEvent e);

}
